unit Unit1; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ExtCtrls, TAGraph, TASeries, TATypes, tineclient, tinetypes, tinedefs;


type

  { TForm1 }

  TForm1 = class(TForm)
    Chart1: TChart;
    CycleTimer: TTimer;
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure CycleTimerTimer(Sender: TObject);
    procedure InitLine();
    procedure InitTineLink();
  private
    { private declarations }
    FLine: TLineSeries;
  public
    { public declarations }
  end; 
const
  BUFFER_LEN = 2000;
var
  Form1: TForm1;
  FloatBuf: array[0 .. (BUFFER_LEN - 1)] of single;   // 4 byte float
  WaveformId: integer;

procedure TineCallback(linkId:integer; completionCode: integer); cdecl;

implementation

uses TAChartUtils;

{ TForm1 }

procedure TineCallback(linkId:integer; completionCode: integer); cdecl;
var index:integer;
begin
  if (completionCode <> 0) then
    begin
    ShowMessage('TRC error: ' +GetLastLinkErrorAsString(completionCode));
    Form1.CycleTimer.Enabled:=false;
    end;
  if linkId = WaveformId then
       begin
       if Form1.FLine <> nil then
          begin
          Form1.Chart1.DeleteSeries(Form1.FLine);
          FreeAndNil(Form1.FLine);
          end;
       Form1.InitLine();
       for index:=0 to (BUFFER_LEN - 1) do
          Form1.FLine.AddXY(index, FloatBuf[index]);
       end;
end;

procedure TForm1.CycleTimerTimer(Sender: TObject);
var retry:boolean;
begin
  { SystemCycle needs to be called regularly. If the return value is true,
    SystemCycle should be called immediately again. }
  retry:=true;
  while retry do
    retry:=SystemCycle(true);
end;

procedure TForm1.InitLine();
begin
  FLine := TLineSeries.Create(Chart1);
  FLine.ShowLines := true;
  FLine.ShowPoints := false;
  FLine.Pointer.Style := psNone;
  FLine.Pointer.Brush.Color := clBlue;
  FLine.Title := 'V_OUT';
  FLine.SeriesColor := clBlue;
  Chart1.AddSeries(FLine);
end;

procedure TForm1.InitTineLink();
var dout : DTYPE;
    DeviceName   : array[0 .. (DEVICE_NAME_SIZE-1)] of char;
    PropertyName : array[0 .. (PROPERTY_NAME_SIZE-1)] of char;
begin
  { initialize an asynchronous link to an array of 2000 4-byte float.
    if data has arrived, the procedure TineCallback is called. }
  {StrPCopy(@DeviceName, 'MTF.RF/MODDATA/KLYMOD2');
  StrPCopy(@PropertyName, 'V_OUT.WAVEFORM');
  } { nvs: instead test with sineserver (sandboxdemo) }
  StrPCopy(@DeviceName, 'TEST/SineServer/SineGen0');
  StrPCopy(@PropertyName, 'Sine');
  dout.dArrayLength := BUFFER_LEN;
  dout.dFormat      := CF_FLOAT;
  dout.data.fptr    := @FloatBuf[0];
  WaveformId := AttachLink(@DeviceName, @PropertyName, @dout, NIL, CA_READ, 1000, @TineCallback, CM_POLL);
  if (WaveformId < 0) then
     ShowMessage('ERROR initializing tine link: "'+GetLastLinkErrorAsString(WaveformId)+'"')
  else
     { start calling SystemCycle periodically }
     CycleTimer.Enabled:=true;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  InitializeTineclientDll('C:\TINE\system32');
  InitTineLink();
end;

procedure TForm1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  case CloseAction of
     caFree:
        begin
        Form1.CycleTimer.Enabled:=false;
        sleep(2*Form1.CycleTimer.Interval);
        UnInitializeTineclientDll();
        end
      else ; { caNone, caHide, caMinimize: do nothing }
  end;
end;

initialization
  {$I unit1.lrs}

end.

