unit tinedefs;
{
   TINE constants for tineclient.pas
   see http://adweb.desy.de/mcs/tine/annotated.html
   interfaces TINE Client API Calls, allow tine client usage in delphi and lazarus
   Tested on:
      - TINE version    4.1.2 and 4.1.3
      - Delphi version  5
      - Lazarus version 0.9.28.2 beta
   Author          Winfried Koehler
   last changed    2010-02-13
}

interface

type
 // type definitions to make c -> delphi conversion easier.
 // NOTE: delphi uses 'string' type, which is incompatible to char * (PChar). PChar needs memory allocation somewhere.
 {a pointer to Char; char *                char   = Char;       PChar   = ^char; }

 {$ifndef LCL}
 {a pointer to PChar; char **           }                       PPchar  = ^Pchar;
 {$endif}
 {unsigned 16-bit                       }  short  = Word;       Pshort  = ^short;
 {signed (= 32-bit)                     }  int    = integer;    Pint    = ^int;
 {signed 8-bit  128 .. 127             }  SINT8  = Shortint;   PSINT8  = ^SINT8;
 {signed 16-bit -32768..32767           }  SINT16 = Smallint;   PSINT16 = ^SINT16;
 {signed 32-bit 2147483648..2147483647 }  SINT32 = Longint;    PSINT32 = ^SINT32;
 {signed 64-bit 2^63..2^631           }  SINT64 = Int64;      PSINT64 = ^SINT64;
 {unsigned 8-bit  0..255                }  UINT8  = Byte;       PUINT8  = ^UINT8;
 {unsigned 16-bit 0..65535              }  UINT16 = Word;       PUINT16 = ^UINT16;
 {unsigned 32-bit 0..4294967295         }  UINT32 = Longword;   PUINT32 = ^UINT32;
 {$ifndef LCL}
 {unsigned 64-bit - no delphi5 support  }  UINT64 = Int64;     // NOTE: needs byte shifting in delphi!!!
 {$endif}
                                           PUINT64 = ^UINT64;

 {4 byte float                          }  float  = Single;     Pfloat  = ^float;
 {$ifndef LCL}
 {8 byte float                             double = Double;   } Pdouble = ^double;
 {$endif}
 {structure type                           struct -> record                       }
 {union type                               union  -> not available                }
 {null pointer. note: NIL is not a type    NULL   = NIL;                          }

const

  ENSTAG                   = 'ENS';
  CSHOSTS                  = 'ENSEQM';
  CSHOSTS_FILE             = 'cshosts.csv';
  FECID_FILE               = 'fecid.csv';
  EXPORT_FILE              = 'exports.csv';
  EQPDB_FILE               = 'eqpdbase.csv';
  FECDB_FILE               = 'fecaddr.csv';
  USERS_FILE               = 'users.csv';
  DENYUSER_FILE            = 'denyuser.csv';
  IPNETS_FILE              = 'ipnets.csv';
  IPBCASTNETS_FILE         = 'ipbcast.csv';
  IPXNETS_FILE             = 'ipxnets.csv';
  IPXBCASTNETS_FILE        = 'ipxbcast.csv';
  ALIAS_FILE               = 'alias.csv';
  FECNAME_UNKNOWN          = '*unknown*';

  HISTORY_HDR_SIZE         = 16;
  SPECTRUM_HDR_SIZE        = 96;
  SPECTRUM_STD_SIZE        = 4096;
  IMAGE_HDR_SIZE           = 188;
  IMAGE_STD_FRAMESIZE      = 500;
  STRING_STD_CAPACITY      = 128;

  //< Maximal length of returned status string
  STATUS_SHORTSIZE         = 32;
  STATUS_SIZE              = 192;

  //< Maximal length of a FEC name.
  FEC_NAME_SIZE            = 16;
  FEC_DESC_SIZE            = 64;
  FEC_LOCATION_SIZE        = 48;
  FEC_VERSION_SIZE         = 16;
  FEC_HDW_SIZE             = 32;
  FEC_RESP_SIZE            = 32;
  FEC_OS_SIZE              = 16;

  //< maximal length of an export name.
  EXPORT_NAME_SHORTSIZE    = 16;
  EXPORT_NAME_SIZE         = 32;
  
  //< Maximal length of a local module name.
  EQM_NAME_SIZE            = 8;
  EQM_NAME_SHORTSIZE       = 6;
  
   //< Maximal length of a property name.
  PROPERTY_NAME_SIZE       = 64;
  PROPERTY_NAME_SHORTSIZE  = 32;
  PROPERTY_DESC_SIZE       = 64;
  PROPERTY_REDIR_SIZE      = STATUS_SIZE;
  
   //< Maximal length of a device name.
  DEVICE_NAME_SIZE         = 64;
  DEVICE_DESC_SIZE         = 64;
  DEVICE_NAME_SHORTSIZE    = 16;
  DEVICE_NAME_MAXSIZE      = 1024;
  DEVICE_REDIR_SIZE        = STATUS_SIZE;
  
   //< Maximal length of a context name.
  CONTEXT_NAME_SHORTSIZE   = 16;
  CONTEXT_NAME_SIZE        = 32;

  SUBSYSTEM_NAME_SIZE      = 16;
  SUBSYSTEM_NAME_SHORTSIZE = 5;

  SET_NAME_SIZE            = 32;

  FORMAT_STR_SIZE          = 64;
  ACCESS_STR_SIZE          = 64;
  ADDR_SIZE                = 16;
  UNITS_SIZE               = 16;
  USERNAME_SIZE            = 16;
  FIELD_NAME_SIZE          = 16;
  TAG_NAME_SIZE            = 16;
  TAG_NAME_SHORTSIZE       = 8;
  HISTORY_HOME_SIZE        = 128;
  FULL_NAME_MAXSIZE        = DEVICE_NAME_MAXSIZE + PROPERTY_NAME_SIZE + EXPORT_NAME_SIZE + CONTEXT_NAME_SIZE;

  ALARM_TAG_SIZE           = 32;
  ALARM_TAG_SHORTSIZE      = 16;

  CLOG_STATUS_NONE         = 0;
  CLOG_STATUS_INFO         = 1;
  CLOG_STATUS_WARN         = 2;
  CLOG_STATUS_ERR          = 3;

  CLOG_PRIORITY_NONE       = 0;
  CLOG_PRIORITY_USEFUL     = 1;
  CLOG_PRIORITY_IMPORTANT  = 2;
  CLOG_PRIORITY_URGENT     = 3;

  //< access lock information/status/setting
  LOCK_UNLOCKED            = 0;
  LOCK_PREEMPTIVE          = 1;
  LOCK_PERSISTENT          = 2;
  LOCK_ABORT               = 3;

  //< error source associated with the input link index
  LNK_ERROR_LCL            = 0;
  LNK_ERROR_RMT            = 1;

  //< the 'reason' (CX_RESPONSE, CX_STALE, etc.) the data have been delivered, see DTYPE
  CX_NULL                  = $0;
  CX_RESPONSE              = $1;
  CX_STALE                 = $2;
  CX_HEARTBEAT             = $4;
  CX_EVENT                 = $8;
  CX_TIMER                 = $10;
  CX_MESSAGE               = $20;
  CX_ERROR                 = $40;
  CX_ATTACH                = $80;
  CX_RETRY                 = $100;


{ uncomment line to use TINE decorated constants. Use only if the usual identifiers
  are already defined somewhere else and no other solution is to be found.
  The constants are the same, but renamed to "TDC_<CONSTANT_NAME>" }
 // {$DEFINE TINE_DECORATED_CONSTANTS}

{$IFDEF TINE_DECORATED_CONSTANTS}
  TDC_CF_DOUBLE             =   512;
  TDC_CF_INT16              =   513;
  TDC_CF_SHORT              =   513;
  TDC_CF_INTEGER            =   513;
  TDC_CF_INT8               =   514;
  TDC_CF_BYTE               =   514;
  TDC_CF_INT32              =   515;
  TDC_CF_LONG               =   515;
  TDC_CF_TEXT               =   516;
  TDC_CF_CHAR               =   516;
  TDC_CF_FLOAT              =   517;
  TDC_CF_INT64              =   518;
  TDC_CF_DLONG              =   518;
  TDC_CF_STRUCT             =   519;
  TDC_CF_CHAR8              =   520;
  TDC_CF_NAME8              =   520;
  TDC_CF_STRING8            =   520;
  TDC_CF_CHAR16             =   521;
  TDC_CF_NAME16             =   521;
  TDC_CF_STRING16           =   521;
  TDC_CF_CHAR16FI           =   522;
  TDC_CF_NAME16FI           =   522;
  TDC_CF_BIT                =   523;
  TDC_CF_HITTS              =   524;
  TDC_CF_CHAR32             =   525;
  TDC_CF_NAME32             =   525;
  TDC_CF_STRING32           =   525;
  TDC_CF_FI                 =   526;
  TDC_CF_FLTINT             =   526;
  TDC_CF_II                 =   527;
  TDC_CF_LI                 =   527;
  TDC_CF_LL                 =   527;
  TDC_CF_INTINT             =   527;
  TDC_CF_LNGINT             =   527;
  TDC_CF_DI                 =   528;
  TDC_CF_DBLINT             =   528;
  TDC_CF_SI                 =   529;
  TDC_CF_SHTINT             =   529;
  TDC_CF_TDS                =   530;
  TDC_CF_IFI                =   530;
  TDC_CF_INTFLTINT          =   530;
  TDC_CF_CHAR48             =   531;
  TDC_CF_NAME48             =   531;
  TDC_CF_STRING48           =   531;
  TDC_CF_CHAR8I             =   532;
  TDC_CF_NAME8I             =   532;
  TDC_CF_CHAR16I            =   533;
  TDC_CF_NAME16I            =   533;
  TDC_CF_CHAR32I            =   534;
  TDC_CF_NAME32I            =   534;
  TDC_CF_CHAR48I            =   535;
  TDC_CF_NAME48I            =   535;
  TDC_CF_USTRING            =   536;
  TDC_CF_XYS                =   537;
  TDC_CF_FFI                =   537;
  TDC_CF_FLTFLTINT          =   537;
  TDC_CF_FII                =   538;
  TDC_CF_FLTINTINT          =   538;
  TDC_CF_FIS                =   538;
  TDC_CF_FILTER             =   539;
  TDC_CF_IFFF               =   539;
  TDC_CF_INTFLTFLTFLT       =   539;
  TDC_CF_SPECTRUM           =   540;
  TDC_CF_ADDRESS            =   541;
  TDC_CF_IIII               =   541;
  TDC_CF_INTINTINTINT       =   541;
  TDC_CF_WINDOW             =   542;
  TDC_CF_TTII               =   542;
  TDC_CF_POINT              =   543;
  TDC_CF_FF                 =   543;
  TDC_CF_XY                 =   543;
  TDC_CF_FLTFLT             =   543;
  TDC_CF_TDSI               =   544;
  TDC_CF_INTFLTSHTSHT       =   544;
  TDC_CF_BOOLEAN            =   545;
  TDC_CF_IIFF               =   546;
  TDC_CF_INTINTFLTFLT       =   546;
  TDC_CF_FWINDOW            =   546;
  TDC_CF_FIFI               =   547;
  TDC_CF_FLTINTFLTINT       =   547;
  TDC_CF_CHAR64             =   548;
  TDC_CF_NAME64             =   548;
  TDC_CF_STRING64           =   548;
  TDC_CF_CHAR64I            =   549;
  TDC_CF_NAME64I            =   549;
  TDC_CF_INTDBL             =   550;
  TDC_CF_LNGDBL             =   550;
  TDC_CF_FLTDBL             =   551;
  TDC_CF_DBLDBL             =   552;
  TDC_CF_NAMEDBLDBL         =   553;
  TDC_CF_NAME16DBLDBL       =   553;
  TDC_CF_NAME16INTINT       =   554;
  TDC_CF_CHAR16INTINT       =   554;
  TDC_CF_NAME16II           =   554;
  TDC_CF_CHAR16II           =   554;
  TDC_CF_STRING16II         =   554;
  TDC_CF_NAMEII             =   554;
  TDC_CF_INTINTINT          =   555;
  TDC_CF_III                =   555;
  TDC_CF_NAME32DBLDBL       =   556;
  TDC_CF_CHAR32DBLDBL       =   556;
  TDC_CF_STRING32DBLDBL     =   556;
  TDC_CF_NAME64DBLDBL       =   557;
  TDC_CF_CHAR64DBLDBL       =   557;
  TDC_CF_STRING64DBLDBL     =   557;
  TDC_CF_IFFFNAME           =   558;
  TDC_CF_IFFFNAME16         =   558;
  TDC_CF_INTFLTFLTFLTNAME16 =   558;
  TDC_CF_UNAME              =   558;
  TDC_CF_XML                =   559;
  TDC_CF_DBLDBLDBL          =   560;
  TDC_CF_NAME64DBLDBLDBL    =   561;
  TDC_CF_CHAR64DBLDBLDBL    =   561;
  TDC_CF_STRING64DBLDBLDBL  =   561;
  TDC_CF_BITFIELD8          =   562;
  TDC_CF_BITFIELD16         =   563;
  TDC_CF_BITFIELD32         =   564;
  TDC_CF_BITFIELD64         =   565;
  TDC_CF_NAME64DBL          =   566;
  TDC_CF_CHAR64DBL          =   566;
  TDC_CF_STRING64DBL        =   566;
  TDC_CF_IMAGE              =   567;
  TDC_CF_HISTORY            =   568;
  TDC_CF_STRING             =   569;
  TDC_CF_ASPECTRUM          =   570;
  TDC_CF_AIMAGE             =   571;
  TDC_CF_DEFAULT            =   766;
  TDC_CF_NULL               =   767;

  TDC_CM_NULL               =    $0;
  TDC_CM_CANCEL             =    $0;
  TDC_CM_SINGLE             =    $1;
  TDC_CM_REFRESH            =    $2;
  TDC_CM_DATACHANGE         =    $2;
  TDC_CM_POLL               =    $3;
  TDC_CM_TIMER              =    $3;
  TDC_CM_REGISTER           =    $4;
  TDC_CM_RECEIVE            =    $5;
  TDC_CM_EVENT              =    $6;
  TDC_CM_NODUPLICATES       =   $10;
  TDC_CM_SEEKPATTERN        =   $20;
  TDC_CM_MISSPATTERN        =   $40;
  TDC_CM_STREAM             =   $80;
  TDC_CM_BCAST              =  $100;
  TDC_CM_NETWORK            =  $200;
  TDC_CM_MCAST              =  $200;
  TDC_CM_USE_ON_ERROR       =  $400;
  TDC_CM_USEONERROR         =  $400;
  TDC_CM_RETRY              =  $800;
  TDC_CM_GROUPED            = $1000;
  TDC_CM_CONNECT            = $2000;
  TDC_CM_WAIT               = $4000;
  TDC_CM_SYNCNOTIFY         = $8000;

  // TINE access codes                                
  TDC_CA_NULL               =    $0;
  TDC_CA_READ               =    $1;
  TDC_CA_WRITE              =    $2;
  TDC_CA_REPEAT             =    $4;
  TDC_CA_SYNC               =    $8;
  TDC_CA_ALARM              =   $10;
  TDC_CA_HIST               =   $20;
  TDC_CA_FIRST              =   $40;
  TDC_CA_LAST               =   $80;
  TDC_CA_RETRY              =  $100;
  TDC_CA_NETWORK            =  $200;
  TDC_CA_QUERY              =  $400;
  TDC_CA_LOCKED             =  $800;
  TDC_CA_STREAM             =  $800;
  TDC_CA_XREAD              =  $800;
  TDC_CA_NORETRY            = $1000;
  TDC_CA_CONNECT            = $2000;
  TDC_CA_MUTABLE            = $4000;
  TDC_CA_SYNCNOTIFY         = $8000;
                                
  TDC_GT_LIN                =     0;
  TDC_GT_LOG                =     1;
  TDC_GT_SLOG               =     2;
  TDC_GT_TD                 =     3;
  TDC_GT_BIT                =     4;
  TDC_GT_TEXT               =     5;
  TDC_GT_ABS                =     6;
  TDC_GT_HEX                =     7;

  TDC_AT_UNKNOWN            =    $0;
  TDC_AT_SCALAR             =    $1;
  TDC_AT_SINGLE             =    $2;
  TDC_AT_DOUBLE             =    $4;
  TDC_AT_BOOLEAN            =    $8;
  TDC_AT_CHANNEL            =   $10;
  TDC_AT_SPECTRUM           =   $20;
  TDC_AT_TRACE              =   $20;
  TDC_AT_WAVEFORM           =   $20;
  TDC_AT_STRUCTURED         =   $40;
  TDC_AT_COLLECTION         =   $40;
  TDC_AT_UNSTRUCTURED       =   $80;
  TDC_AT_BLOB               =   $80;
  TDC_AT_IMAGE              =   $80;
  TDC_AT_JPEG               =  $100;
  TDC_AT_MPEG               =  $200;
  TDC_AT_BMP                =  $400;
  TDC_AT_GIF                =  $800;
  TDC_AT_WAV                = $1000;
  TDC_AT_MP3                = $2000;

// image format definitions
  TDC_IF_GRAY               =     0;
  TDC_IF_RGB                =     1;
  TDC_IF_RGBA               =     2;
  TDC_IF_RGB15              =     3;
  TDC_IF_RGB16              =     4;
  TDC_IF_JPEG               =     5;
  TDC_IF_TIFF               =     6;
  TDC_IF_YUV411             =     7;
  TDC_IF_YUV422             =     8;
  TDC_IF_YUV444             =     9;
  TDC_IF_HUFFYUV            =    10;
  TDC_IF_BITMAP             =    11;
                        
// image flags
  TDC_IFLG_LITTLE_ENDIAN    =    $1;
  TDC_IFLG_LOSSLESS_COMP    =    $2;
  TDC_IFLG_HORIZONTAL_FLIP  =    $4;
  TDC_IFLG_VERTICAL_FLIP    =    $8;
  
// property signals
  TDC_PS_NONE               =    $0; ///< NULL signal
  TDC_PS_ACCESS             =    $1; ///< property is being accessed by a caller
  TDC_PS_RETRY              =    $2; ///< property is being retried
  TDC_PS_LATE               =    $4; ///< property is being returned late
  TDC_PS_PENDING            =    $8; ///< property is being called while last transmission still pending
  TDC_PS_SENT               =   $10; ///< property has been sent to caller
  TDC_PS_ALL                =   $ff; ///< signal on any event

{$ELSE}

  CF_DOUBLE             =   512;
  CF_INT16              =   513;
  CF_SHORT              =   513;
  CF_INTEGER            =   513;
  CF_INT8               =   514;
  CF_BYTE               =   514;
  CF_INT32              =   515;
  CF_LONG               =   515;
  CF_TEXT               =   516;
  CF_CHAR               =   516;
  CF_FLOAT              =   517;
  CF_INT64              =   518;
  CF_DLONG              =   518;
  CF_STRUCT             =   519;
  CF_CHAR8              =   520;
  CF_NAME8              =   520;
  CF_STRING8            =   520;
  CF_CHAR16             =   521;
  CF_NAME16             =   521;
  CF_STRING16           =   521;
  CF_CHAR16FI           =   522;
  CF_NAME16FI           =   522;
  CF_BIT                =   523;
  CF_HITTS              =   524;
  CF_CHAR32             =   525;
  CF_NAME32             =   525;
  CF_STRING32           =   525;
  CF_FI                 =   526;
  CF_FLTINT             =   526;
  CF_II                 =   527;
  CF_LI                 =   527;
  CF_LL                 =   527;
  CF_INTINT             =   527;
  CF_LNGINT             =   527;
  CF_DI                 =   528;
  CF_DBLINT             =   528;
  CF_SI                 =   529;
  CF_SHTINT             =   529;
  CF_TDS                =   530;
  CF_IFI                =   530;
  CF_INTFLTINT          =   530;
  CF_CHAR48             =   531;
  CF_NAME48             =   531;
  CF_STRING48           =   531;
  CF_CHAR8I             =   532;
  CF_NAME8I             =   532;
  CF_CHAR16I            =   533;
  CF_NAME16I            =   533;
  CF_CHAR32I            =   534;
  CF_NAME32I            =   534;
  CF_CHAR48I            =   535;
  CF_NAME48I            =   535;
  CF_USTRING            =   536;
  CF_XYS                =   537;
  CF_FFI                =   537;
  CF_FLTFLTINT          =   537;
  CF_FII                =   538;
  CF_FLTINTINT          =   538;
  CF_FIS                =   538;
  CF_FILTER             =   539;
  CF_IFFF               =   539;
  CF_INTFLTFLTFLT       =   539;
  CF_SPECTRUM           =   540;
  CF_ADDRESS            =   541;
  CF_IIII               =   541;
  CF_INTINTINTINT       =   541;
  CF_WINDOW             =   542;
  CF_TTII               =   542;
  CF_POINT              =   543;
  CF_FF                 =   543;
  CF_XY                 =   543;
  CF_FLTFLT             =   543;
  CF_TDSI               =   544;
  CF_INTFLTSHTSHT       =   544;
  CF_BOOLEAN            =   545;
  CF_IIFF               =   546;
  CF_INTINTFLTFLT       =   546;
  CF_FWINDOW            =   546;
  CF_FIFI               =   547;
  CF_FLTINTFLTINT       =   547;
  CF_CHAR64             =   548;
  CF_NAME64             =   548;
  CF_STRING64           =   548;
  CF_CHAR64I            =   549;
  CF_NAME64I            =   549;
  CF_INTDBL             =   550;
  CF_LNGDBL             =   550;
  CF_FLTDBL             =   551;
  CF_DBLDBL             =   552;
  CF_NAMEDBLDBL         =   553;
  CF_NAME16DBLDBL       =   553;
  CF_NAME16INTINT       =   554;
  CF_CHAR16INTINT       =   554;
  CF_NAME16II           =   554;
  CF_CHAR16II           =   554;
  CF_STRING16II         =   554;
  CF_NAMEII             =   554;
  CF_INTINTINT          =   555;
  CF_III                =   555;
  CF_NAME32DBLDBL       =   556;
  CF_CHAR32DBLDBL       =   556;
  CF_STRING32DBLDBL     =   556;
  CF_NAME64DBLDBL       =   557;
  CF_CHAR64DBLDBL       =   557;
  CF_STRING64DBLDBL     =   557;
  CF_IFFFNAME           =   558;
  CF_IFFFNAME16         =   558;
  CF_INTFLTFLTFLTNAME16 =   558;
  CF_UNAME              =   558;
  CF_XML                =   559;
  CF_DBLDBLDBL          =   560;
  CF_NAME64DBLDBLDBL    =   561;
  CF_CHAR64DBLDBLDBL    =   561;
  CF_STRING64DBLDBLDBL  =   561;
  CF_BITFIELD8          =   562;
  CF_BITFIELD16         =   563;
  CF_BITFIELD32         =   564;
  CF_BITFIELD64         =   565;
  CF_NAME64DBL          =   566;
  CF_CHAR64DBL          =   566;
  CF_STRING64DBL        =   566;
  CF_IMAGE              =   567;
  CF_HISTORY            =   568;
  CF_STRING             =   569;
  CF_ASPECTRUM          =   570;
  CF_AIMAGE             =   571;
  CF_DEFAULT            =   766;
  CF_NULL               =   767;

  // data acquisition modes. see http://adweb.desy.de/mcs/tine/tinemodes.html  
  CM_CANCEL             =    $0; CM_NULL       = CM_CANCEL;       // cancels an active data link
  CM_SINGLE             =    $1;                                  // One-time non-persistent data acquisition
  CM_DATACHANGE         =    $2; CM_REFRESH    = CM_DATACHANGE;   // Persistent data acquisition with data transfer on change of data (zero-tolerance data comparison) 
  CM_POLL               =    $3; CM_TIMER      = CM_POLL;         // persistent data acquisition at the specified polling rate
  CM_REGISTER           =    $4;                                  // used primarily internally, Registers the data acquisition request in a client's connection table
  CM_RECEIVE            =    $5;                                  // The data link is not bound to and endpoint, but expects to receive global information. AttachLink() + CM_RECEIVE is similar to recvNetGlobal()
  CM_EVENT              =    $6;                                  // Presistent data acquisition with data transfer and notification upon signalled events from the server.
  CM_NODUPLICATES       =   $10;                                  // Instructs the client engines to suppress event (i.e. callback) notification if the incoming data do not change within the given tolerance (see: SetNotificationTolerance()).
  CM_SEEKPATTERN        =   $20;
  CM_MISSPATTERN        =   $40;
  CM_STREAM             =   $80;                                  // the data acquisition should occur via a connected socket, i.e. using a TCP/IP stream instead of UDP datagrams
  CM_BCAST              =  $100;                                  // Apply this flag to signal the server to send the data as a broadcast on the 'control' network, see ipbcast.csv
  CM_NETWORK            =  $200; CM_MCAST      = CM_NETWORK;      // Apply this flag to signal the server to send the data as a multicast
  CM_USE_ON_ERROR       =  $400; CM_USEONERROR = CM_USE_ON_ERROR; // Apply this flag to signal the client engine to replace faulty data (non-zero completion code) with initial (i.e. pre-link) contents of the output data buffer.
  CM_RETRY              =  $800;                                  // Used internally only
  CM_GROUPED            = $1000;                                  // client: call the callback routine only once when the data for all links with the same callback function has arrived
  CM_CONNECT            = $2000;                                  // data acquisition should occur via a connected socket, i.e. using TCP/IP blocks instead of UDP datagrams
  CM_WAIT               = $4000;                                  // signal the client code execution to wait for the first data acquisition to complete before continuing
  CM_SYNCNOTIFY         = $8000;                                  // Apply this flag to signal the client engine should call its callback routines as the data come in. Normally, the callback nofitication is deferred until all incoming data is processed.
                                                                  // If this flag is used, you CANNOT use synchronous links (e.g. ExecLinkEx()) inside a callback routine
  CM_REMOVEVOLATILE     = CM_NODUPLICATES OR CM_SEEKPATTERN OR CM_MISSPATTERN OR CM_USEONERROR OR CM_GROUPED OR CM_WAIT;
 
  // TINE access codes
  CA_NULL               =    $0;                                  // Used internally
  CA_READ               =    $1;                                  // requesting read access, will be applied by default. visible to both client and server
  CA_WRITE              =    $2;                                  // requesting write access. Is checked against a server's users.csv and allowed networks list (ipnets.csv and/or ipxnets.csv).
  CA_REPEAT             =    $4;                                  // servers: Set by the server prior to calling the equipment module. The call was previously instructed to retry the link via the return code not_ready
  CA_SYNC               =    $8;                                  // servers: Set by the server prior to calling the equipment module. The call is being made on behalf of a non-peristent synchronous data link.
  CA_ALARM              =   $10;                                  // servers: Set by the server prior to calling the equipment module. The call is being made on behalf of the local alarm server (due to the Alarm Watch Table).
  CA_HIST               =   $20;                                  // servers: Set by the server prior to calling the equipment module. The call is being made on behalf of the local history server
  CA_FIRST              =   $40;                                  // servers: Set by the server prior to calling the equipment module. The call is coming into scope. For instance a persistent data link for a particular contract has just been established.
  CA_LAST               =   $80;                                  // servers: Set by the server prior to calling the equipment module. The call is going out of scope. For instance a persistent data link for a particular contract has just been canceled. Synchronous calls will set the CA_FIRST bit along with the CA_LAST bit as well as the CA_SYNC bit.
  CA_RETRY              =  $100;                                  // clients: in conjunction with access mode CM_SINGLE only. It instructs the subsystem to retry the data acquistion in case of connection timeout or link error
  CA_NETWORK            =  $200;                                  // clients: same effect as the mode flag CM_NETWORK. Is useful for API interfaces such as ExecLinkEx() which does not take a 'mode' parameter
  CA_QUERY              =  $400;                                  // servers: Is set prior to calling an equipment module if the call is a query call coming from the TINE kernel
  CA_LOCKED             =  $800;                                  // servers: This access bit is set if a client has an access lock token on the server and issues a WRITE request (command). The equipment module can check to see if this bit set when processing a WRITE command and (for instance) refuse the command if there is no access lock token.
  CA_STREAM             =  $800;                                  // clients: same effect as the mode flag CM_STREAM. Is useful for API interfaces such as ExecLinkEx() which does not take a 'mode' parameter.
  CA_XREAD              =  $800;
  CA_NORETRY            = $1000;                                  // clients: instructs the subsystem not to re-issue the link upon timeout error, regardless of the 'gAlwaysRetry' setting.
  CA_CONNECT            = $2000;                                  // clients: same effect as the mode flag CM_CONNECT. Is useful for API interfaces such as ExecLinkEx() which does not take a 'mode' parameter
  CA_MUTABLE            = $4000;                                  // clients: Applies only to synchronous calls ExecLink() or ExecLinkEx(). This allows the subsystem to adjust the output data object to account for the amount of data actually returned by the server. This is the only thread-safe way of obtaining this information with a sychronous call. Note that calling GetCompletionDataSize(-1) is not thread-safe
  CA_SYNCNOTIFY         = $8000;                                  // clients: same effect as the mode flag CM_SYNCNOTIFY
                                
  GT_LIN                =     0;
  GT_LOG                =     1;
  GT_SLOG               =     2;
  GT_TD                 =     3;
  GT_BIT                =     4;
  GT_TEXT               =     5;
  GT_ABS                =     6;
  GT_HEX                =     7;

  // List of data array type flags                              
  AT_UNKNOWN            =    $0;                                  // either unspecified or the property is question does not involve a transfer of data.
  AT_SCALAR             =    $1;                                  // not an array, but a single value.
  AT_SINGLE             =    $2;                                  // a single dimensioned array (i.e. a vector).
  AT_DOUBLE             =    $4;                                  // a double dimensioned array (a matrix).
  AT_BOOLEAN            =    $8;                                  // a boolean value (TRUE or FALSE)
  AT_CHANNEL            =   $10;                                  // an array representing a multi-channel array, where the array elements represent the associated property values of corresponding device modules. Typically the property is overloaded to return the group of devices beginning with and specified by the device name used in the call and to fill in the requested array up to the length specified. Hence the entire group can be obtained, or a single device, or a subset of the array property so registered. Such arrays are best displayed as a historgram.
  AT_SPECTRUM           =   $20; AT_TRACE      = AT_SPECTRUM;     // an array representing a trace or waveform. Such arrays are best displayed as a line trace. The horizontal display information concerning the range and units can likewise be obtained with a call for extended property query information                                 
                                 AT_WAVEFORM   = AT_SPECTRUM;
  AT_STRUCTURED         =   $40; AT_COLLECTION = AT_STRUCTURED;   // an array whose elements represent non-plottable information unrelated to a common set of engineering units. For instance an array of long integers where the first gives an number, the second a timestamp, the third a target value, and so on.                                 
  AT_UNSTRUCTURED       =   $80; AT_BLOB       = AT_UNSTRUCTURED; // an array whose elements are unstructured and are to be manipulated (and understood) by the caller. For instance an image is usually sent and a unstructured byte-blob which needs to be decompressed by the client, before being displayed.
                                 AT_IMAGE      = AT_UNSTRUCTURED;
  AT_JPEG               =  $100;                                  // byte blob representing a JPEG image
  AT_MPEG               =  $200;                                  // byte blob representing a MPEG image
  AT_BMP                =  $400;                                  // byte blob representing a bitmap image
  AT_GIF                =  $800;                                  // byte blob representing a GIF image
  AT_WAV                = $1000;                                  // byte blob representing a WAV audio stream
  AT_MP3                = $2000;                                  // byte blob representing a MP3 audio stream

// image format definitions
  IF_GRAY               =     0;
  IF_RGB                =     1;
  IF_RGBA               =     2;
  IF_RGB15              =     3;
  IF_RGB16              =     4;
  IF_JPEG               =     5;
  IF_TIFF               =     6;
  IF_YUV411             =     7;
  IF_YUV422             =     8;
  IF_YUV444             =     9;
  IF_HUFFYUV            =    10;
  IF_BITMAP             =    11;
                        
// image flags
  IFLG_LITTLE_ENDIAN    =    $1;
  IFLG_LOSSLESS_COMP    =    $2;
  IFLG_HORIZONTAL_FLIP  =    $4;
  IFLG_VERTICAL_FLIP    =    $8;
  
// property signals
  PS_NONE               =    $0; ///< NULL signal
  PS_ACCESS             =    $1; ///< property is being accessed by a caller
  PS_RETRY              =    $2; ///< property is being retried
  PS_LATE               =    $4; ///< property is being returned late
  PS_PENDING            =    $8; ///< property is being called while last transmission still pending
  PS_SENT               =   $10; ///< property has been sent to caller
  PS_ALL                =   $ff; ///< signal on any event

{$ENDIF}

// are these functions needed at all? hmm..
function LFMT                       (fmt:integer):integer;
function BFMT                       (fmt:integer):integer;
function getFormatHeaderSize        (fmt:integer):integer;
function getFormatDataType          (fmt:integer):integer;
function getFormatBodyPtr           (dst:pointer; fmt:integer):pointer;
function getFormatBodyPtrAddr       (dst:pointer; fmt:integer):pointer;
function getFormatBodyBufferSize    (dst:pointer; fmt:integer):UINT32;
function getFormatBodyBufferSizeAddr(dst:pointer; fmt:integer):pointer;
function getFormatTypeSize          (fmt:integer):integer;
function fmtsizeof                  (fmt:integer):integer;
function getFormatStandardCapacity  (fmt:integer):integer;
function isPrimitiveFormat          (fmt:integer):boolean; //NOTE: return type delphi boolean, not integer.

implementation

uses tinetypes, tineclient
{$IfDef WINDOWS}
     , windows
{$ENDIF}
;

function getFormatHeaderSize(fmt:integer):integer;
begin
 case fmt of
   CF_SPECTRUM, CF_ASPECTRUM:
      result:=SPECTRUM_HDR_SIZE;
   CF_IMAGE, CF_AIMAGE:
      result:=IMAGE_HDR_SIZE;
   else
      result:=0;
 end;
end;

function getFormatDataType(fmt:integer):integer;
begin
 case fmt of
   CF_SPECTRUM, CF_ASPECTRUM:
       result:=CF_FLOAT;
   CF_IMAGE, CF_AIMAGE:
       result:=CF_BYTE;
   else
       result:=0;
 end;
end;

function getFormatBodyPtr(dst:pointer; fmt:integer):pointer;
   function getSPECTRUMspectbuffer(p:pASPECTRUM):Pfloat; begin result:=Addr(p^.spectBuffer); end;
   function getDIMAGEframebuffer  (p:pDIMAGE)   :Pbyte;  begin result:=Addr(p^.frameBuffer); end;
begin
  case fmt of
    CF_IMAGE, CF_AIMAGE:
      result := getDIMAGEframebuffer(dst);
    CF_ASPECTRUM:
      result := getSPECTRUMspectbuffer(dst);
    else
      result:=NIL;
  end;
end;

function getFormatBodyPtrAddr(dst:pointer; fmt:integer):pointer;
var pDi:pDIMAGE;
    pAs:pASPECTRUM;
begin
  case fmt of
    CF_IMAGE, CF_AIMAGE:
      begin
      pDi := dst;
      result := Addr(pDi^.frameBuffer);
      end;
    CF_ASPECTRUM:
      begin
      pAs := dst;
      result := Addr(pAs^.spectBuffer);
      end;
    else
      result:=NIL;
  end;
end;

function getFormatBodyBufferSize(dst:pointer; fmt:integer):UINT32;
   function getSPECTRUMspectbuffersize(p:pASPECTRUM):UINT32; begin result:=p^.spectBufferSize; end;
   function getDIMAGEframebuffersize  (p:pDIMAGE)   :UINT32; begin result:=p^.frameBufferSize; end;
begin
  case fmt of
    CF_IMAGE, CF_AIMAGE:
      result := getDIMAGEframebuffersize(dst);
    CF_ASPECTRUM:
      result := getSPECTRUMspectbuffersize(dst);
    else
      result:=0;
  end;
end;

function getFormatBodyBufferSizeAddr(dst:pointer; fmt:integer):pointer;
var pDi:pDIMAGE;
    pAs:pASPECTRUM;
begin
  case fmt of
    CF_IMAGE, CF_AIMAGE:
      begin
      pDi := dst;
      result := Addr(pDi^.frameBufferSize);
      end;
    CF_ASPECTRUM:
      begin
      pAs := dst;
      result := Addr(pAs^.spectBufferSize);
      end;
    else
      result:=NIL;
  end;
end;

function fmtsizeof(fmt:integer):integer;
begin
  result:=GetFormatSize(( fmt mod 256) + CF_DOUBLE);
end;

function getFormatTypeSize(fmt:integer):integer;
begin
  case fmt of
    CF_IMAGE, CF_AIMAGE:   result:=sizeof(DIMAGE);
    CF_ASPECTRUM:          result:=sizeof(ASPECTRUM);
    CF_SPECTRUM:           result:=sizeof(DSPECTRUM);
    else
      result:=fmtsizeof(fmt);
  end;
end;

function getFormatStandardCapacity(fmt:integer):integer;
begin
  case fmt of
    CF_IMAGE, CF_AIMAGE:       result:=IMAGE_STD_FRAMESIZE;
    CF_ASPECTRUM, CF_SPECTRUM: result:=SPECTRUM_STD_SIZE;
    CF_STRING:                 result:=IMAGE_STD_FRAMESIZE;
    else
      result:=1;
  end;
end;

function isPrimitiveFormat(fmt:integer):boolean;
begin
  if fmt <= CF_INT64 then
     result:=TRUE
  else
     result:=FALSE;
end;

function LFMT(fmt : integer): integer;
begin
  result:= (fmt mod 256) + CF_DOUBLE;
end;

function BFMT(fmt : integer): integer;
begin
  result := (fmt mod 256);
end;

end.
